#
# Copyright (c) 2012-2024 Snowflake Computing Inc. All rights reserved.
#

__all__ = ["SnowConnection"]

from snowflake.snowpark_checkpoints_collector.snow_connection_model.snow_connection import (
    SnowConnection,
)
