#
# Copyright (c) 2012-2024 Snowflake Computing Inc. All rights reserved.
#

__all__ = [
    "ColumnCollectorManager",
]

from snowflake.snowpark_checkpoints_collector.column_collection.column_collector_manager import (
    ColumnCollectorManager,
)
