#
# Copyright (c) 2012-2024 Snowflake Computing Inc. All rights reserved.
#

__all__ = ["PanderaColumnChecksManager"]

from snowflake.snowpark_checkpoints_collector.column_pandera_checks.pandera_column_checks_manager import (
    PanderaColumnChecksManager,
)
